# Local Imports
from .DocxFormField import DocxFormField

try:
    from constants import XML_PREFIX
    from type_aliases import Element
except ImportError:
    from ..constants import XML_PREFIX
    from ..type_aliases import Element


class TextFormField(DocxFormField):
    """
    This class holds all information & functions regarding a text form field.
    """

    def __init__(self, root: Element, file_path: str, name: str) -> None:
        super.__init__(root, file_path, name)
        self.value = self.__get_text()

    def __get_text(self) -> str:
        """
        Return the value of each text tag concatenated
        """
        child: Element
        tag_text: str = ""
        for child in self.root.iter(f"{XML_PREFIX}t"):
            tag_text += child.text

        return tag_text



"""
This file contains any constants used in the docx-form module.
"""

XML_PREFIX = "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}"
XML_CHECK = "{http://schemas.microsoft.com/office/word/2010/wordml}"



# Package Imports
from lxml import etree

# Local Imports
from .DocxContentControl import DocxContentControl

try:
    from type_aliases import Element
    from constants import XML_PREFIX
    from globals import Raw_XML
except ImportError:
    from ..type_aliases import Element
    from ..constants import XML_PREFIX
    from ..globals import Raw_XML


class ComboBoxOption:
    """
    This class contains all properties and methods for a ComboBox list item.

    :param str display_text: The display text of the option
    :param str value: The value of the option
    """

    def __init__(self, display_text: str, value: str):
        self.display_text = display_text
        self.value = value


class ComboBoxContentControl(DocxContentControl):
    """
    This class contains all properties and methods for a ComboBox content control.

    :param DocxContentControl: This class extends the DocxContentControl class
    """

    def __init__(self, root: Element, file_path: str):
        super().__init__(root, file_path)
        self.options = self.__get_options()

    def __get_options(self) -> list[ComboBoxOption] | None:
        """
        This method gets the options of a ComboBox content control.

        :return: The options of the ComboBox content control
        :rtype: list[ComboBoxOption] | None
        """
        # The root of the document
        root: Element = etree.XML(Raw_XML.raw_xml)
        # The body of the document
        body: Element = root.getchildren()[0]

        element: Element
        for element in body.iter(f"{XML_PREFIX}sdt"):
            # Get the nested id tag
            id_tag: Element
            for id_tag in element.iter(f"{XML_PREFIX}id"):
                # If the id matches
                if id_tag.get(f"{XML_PREFIX}val") == self.id:
                    # Get the options
                    options: list[ComboBoxOption] = []
                    option: Element
                    for option in element.iter(f"{XML_PREFIX}listItem"):
                        tag_info = ComboBoxOption(
                            option.get(f"{XML_PREFIX}displayText"),
                            option.get(f"{XML_PREFIX}value"),
                        )
                        options.append(tag_info)

                    return options

        return None

    def print_options(self) -> None:
        """
        This method prints the options of a ComboBox content control.
        """
        if self.options is not None:
            index = 0
            for option in self.options:
                print(
                    f'{index}: Display Value = "{option.display_text}" || Value = "{option.value}"'
                )
                index += 1
        else:
            print("No options found")

    def set_text(self, option_index: int) -> None:
        """
        This method sets the text of a ComboBox content control.

        :param int option_index: The index of the new text to set
        """

        # The root of the document
        root: Element = etree.XML(Raw_XML.raw_xml)
        # The body of the document
        body: Element = root.getchildren()[0]

        element: Element
        for element in body.iter(f"{XML_PREFIX}sdt"):
            # Get the nested id tag
            id_tag: Element
            for id_tag in element.iter(f"{XML_PREFIX}id"):
                # If the id matches
                if id_tag.get(f"{XML_PREFIX}val") == self.id:
                    # Replace the text
                    content_tag: Element = element.find(f"{XML_PREFIX}sdtContent")

                    # TODO: Test this in case it is not a universal index
                    # Dig to the t tag
                    p_tag: Element = content_tag.getchildren()[0]
                    r_tag: Element = p_tag.getchildren()[0]
                    t_tag: Element = r_tag.getchildren()[0]

                    # Replace the old text with the new option
                    t_tag.text = self.options[option_index].value

        # Write the new document to raw_xml
        Raw_XML.raw_xml = etree.tostring(root)



# Package Imports
from zipfile import ZipFile
from lxml import etree
import re, os

try:
    from enums import TagType
    from content_controls import (
        PlainTextContentControl,
        RichTextContentControl,
        ComboBoxContentControl,
        DropDownListContentControl,
        CheckBoxContentControl,
        DatePickerContentControl,
    )
    from form_fields import TextFormField, CheckBoxFormField, DropDownFormField
    from constants import XML_PREFIX, XML_CHECK
    from globals import Raw_XML
    from type_aliases import Element
except ImportError:
    from .enums import TagType
    from .content_controls import (
        PlainTextContentControl,
        RichTextContentControl,
        ComboBoxContentControl,
        DropDownListContentControl,
        CheckBoxContentControl,
        DatePickerContentControl,
    )
    from .form_fields import TextFormField, CheckBoxFormField, DropDownFormField
    from .constants import XML_PREFIX, XML_CHECK
    from .globals import Raw_XML
    from .type_aliases import Element

# Local Type Aliases
ContentControl = (
    CheckBoxContentControl
    | RichTextContentControl
    | ComboBoxContentControl
    | DatePickerContentControl
    | DropDownListContentControl
    | PlainTextContentControl
)
FormField = CheckBoxFormField | DropDownFormField | TextFormField


class DocxForm:
    """
    This is the one and only entry point for the DocxForm package.
    DocxForm holds all content controls for a given document and allows read and write operations on the document.
    """

    def __init__(self, file_path: str):
        """
        This initalizes the DocxForm class.

        :param str file_path: The path to the document
        """

        self.file_path: str = self.__verify_path(file_path)
        Raw_XML.raw_xml = self.__get_raw_xml()
        self.content_control_forms_and_form_fields: list[
            ContentControl | FormField
        ] = self.__get_all_content_control_forms_and_form_fields()

    def save(self, destination_path: str | None = None):
        """
        This method saves the document to the destination path if a path is given.
        If not, the original file is overwritten.
        :param str destination_path: The full path to save the file to, defaults to None
        """

        # If no name is given the original docx will be overwritten
        if (
            destination_path == None
            or destination_path == " "
            or destination_path == ""
        ):
            temp_path = self.file_path.replace(".docx", "-temp.docx")

            with ZipFile(self.file_path, "a") as doc, ZipFile(
                temp_path, "w"
            ) as temp_doc:
                # Copy all contents except the "word/document.xml" file from the docx to the temp docx
                doc_list = doc.infolist()
                for item in doc_list:
                    if item.filename != "word/document.xml":
                        temp_doc.writestr(item, doc.read(item.filename))
                # Write changes to new docx
                temp_doc.writestr("word/document.xml", Raw_XML.raw_xml)

            # Delete the original docx
            os.remove(self.file_path)

            # Rename the temporary docx to match the original name
            os.renames(temp_path, self.file_path)

        # Saves to a new file -- uses path as destination, relative path does work
        else:
            # Replace document path with the destination path
            new_path = destination_path

            with ZipFile(self.file_path, "a") as old_doc, ZipFile(
                new_path, "w"
            ) as new_doc:
                # Copy all contents except the "word/document.xml" file from the old docx to the new docx
                doc_list = old_doc.infolist()
                for item in doc_list:
                    if item.filename != "word/document.xml":
                        new_doc.writestr(item, old_doc.read(item.filename))
                # Write changes to new docx
                new_doc.writestr("word/document.xml", Raw_XML.raw_xml)

    def list_all_content_controls_and_form_fields(self):
        """
        This method prints all content controls and form fields in the document.
        """

        # Track current array index in for loop
        pos = 0
        # print values from each control with the index in content_control_forms
        for control in self.content_control_forms_and_form_fields:
            if isinstance(control, ContentControl):
                print(
                    str(pos)
                    + ": "
                    + control.__class__.__name__
                    + " | id: "
                    + control.id
                    + " | text: "
                    + control.text
                )
            else:
                print(
                    str(pos)
                    + ": "
                    + control.__class__.__name__
                    + " | name: "
                    + control.name
                    + " | value: "
                    + control.value
                )
            pos = pos + 1

    def __get_raw_xml(self) -> str:
        """
        This method returns the raw xml of the document.

        :return str: The raw xml of the document
        """
        with ZipFile(self.file_path) as document:
            # Put the raw xml into an xml file for testing
            full_path = ""
            if len(full_path) > 0:
                with open(full_path, "wb") as f:
                    f.write(document.read("word/document.xml"))

            return document.read(
                "word/document.xml"
            )  # .decode("utf-8") this creates problems for some reason

    def __verify_path(self, file_path: str) -> str:
        """
        This method verifies that the file path is valid.

        :param str file_path: The path to the document
        :raises Exception: If the file path is not to a .docx file
        :return str: The file path
        """
        # regex to check for docx extension in file path
        verify = re.compile("\\.docx$")

        if not verify.search(file_path):
            raise Exception("File is not docx")

        # File path verify with file open attempt

        try:
            open(file_path, "a")

        except OSError:
            ...

            # Zipfile ops at self.file_path and open xml

        with ZipFile(file_path) as myzip:
            with myzip.open("word/document.xml") as myfile:
                myfile.read()

        return file_path

    def __get_all_content_control_forms_and_form_fields(self) -> list[ContentControl]:
        """
        This method returns all content control forms in the document.

        :return list[ContentControl]:
        """
        with ZipFile(self.file_path) as document:
            # Return variable
            content_control_forms: list[ContentControl] = []
            # The root of the document
            root: Element = etree.XML(document.read("word/document.xml"))
            # The body of the document
            body: Element = root.getchildren()[0]

            # Find all content controls in the document
            content_control: Element
            for content_control in body.iter(f"{XML_PREFIX}sdt"):
                content_control_forms.append(
                    self.__determine_content_control(content_control, TagType.SDT)
                )

            # Find all form fields in the document
            p_tag: Element
            for p_tag in body.iter(f"{XML_PREFIX}p"):
                """
                Note: The same <w:p> tag can be used for multiple checkboxes depending on the document.
                The name value will differentiate them.
                """
                child: Element
                for child in p_tag.iter(f"{XML_PREFIX}name"):
                    content_control_forms.append(
                        self.__determine_content_control(
                            p_tag, TagType.P, child.get(f"{XML_PREFIX}val")
                        )
                    )

            # Remove the None values from the list ()
            content_control_forms = [x for x in content_control_forms if x is not None]

            return content_control_forms

    def __determine_content_control(
        self, parent_tag: Element, tag_type: TagType, name: str = ""
    ) -> ContentControl | FormField | None:
        """
        This method determines the type of content control and returns the appropriate object.

        :param Element parent_tag: The parent tag of the content control. This will be a <w:sdt> tag or a <w:p> tag.
        :param TagType tag_type: The type of tag that the content control is in. This will be either SDT or P.
        :param str name: The name of the content control. This is only used for form fields.
        :return: The content control or form field object
        :rtype: ContentControl | FormField | None
        """

        child: Element
        match tag_type:
            # This case handles all Content Control types
            case TagType.SDT:
                # Find a Date Picker Content Control if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}date"):
                    return DatePickerContentControl(parent_tag, self.file_path)

                # Find a Drop Down List Content Control if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}dropDownList"):
                    return DropDownListContentControl(parent_tag, self.file_path)

                # Find a Combo Box Content Control if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}comboBox"):
                    return ComboBoxContentControl(parent_tag, self.file_path)

                # Find a Plain Text Content Control if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}text"):
                    return PlainTextContentControl(parent_tag, self.file_path)

                # Find a Check Box Content Control if it exists
                for child in parent_tag.iter(f"{XML_CHECK}checkbox"):
                    return CheckBoxContentControl(parent_tag, self.file_path)

                # Otherwise, it is a Rich Text Content Control
                return RichTextContentControl(parent_tag, self.file_path)

            # This case handles all Form Fields
            case TagType.P:
                # Find a Text Form Field if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}textInput"):
                    return TextFormField(parent_tag, self.file_path, name)

                # Find a Drop Down Form Field if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}ddList"):
                    return DropDownFormField(parent_tag, self.file_path, name)

                # Find a Check Box Form Field if it exists
                for child in parent_tag.iter(f"{XML_PREFIX}checkBox"):
                    # return CheckBoxFormField(parent_tag, self.file_path, name)
                    # TODO: Find a way to differentiate checkboxes from each other reliably
                    return None


# Use this for debugging, then move to a test file.
# This will run if you run this file directly.
if __name__ == "__main__":
    ...



# Local Imports
from .DocxFormField import DocxFormField

try:
    from constants import XML_PREFIX
    from type_aliases import Element
except ImportError:
    from ..constants import XML_PREFIX
    from ..type_aliases import Element


class DropDownFormField(DocxFormField):
    """
    This class holds all information & functions regarding a drop down form field.
    """

    def __init__(self, root: Element, file_path: str, name: str) -> None:
        super.__init__(root, file_path, name)
        self.value = "Not currently supported"
        self.entries = self.__get_entries()

    def __get_entries(self) -> list[str]:
        """
        Return the value of each listEntry tag's val attribute
        """
        entries: list[str] = []
        child: Element
        for child in self.root.iter(f"{XML_PREFIX}listEntry"):
            entries.append(child.attrib[f"{XML_PREFIX}val"])

        return entries



# Package Imports
from datetime import datetime
from lxml import etree

# Local Imports
from .DocxContentControl import DocxContentControl

try:
    from type_aliases import Element
    from constants import XML_PREFIX
    from globals import Raw_XML
except ImportError:
    from ..type_aliases import Element
    from ..constants import XML_PREFIX
    from ..globals import Raw_XML


class DatePickerContentControl(DocxContentControl):
    """
    This class contains all properties and methods for a DatePicker content control.

    :param DocxContentControl: This class extends the DocxContentControl class
    """

    def __init__(self, root: Element, file_path: str):
        super().__init__(root, file_path)
        (
            full_date,
            date_format,
            language_id,
            calendar_type,
        ) = self.__get_date_picker_information()
        self.full_date = full_date
        self.date_format = date_format
        self.language_id = language_id
        self.calendar_type = calendar_type

    def __get_date_picker_information(
        self,
    ) -> tuple[str | None, str | None, str | None, str | None]:
        """
        This method gets the date picker information of a DatePicker content control.
        The information is returned as a tuple in the following order:
        (full_date, date_format, date, time)

        :return: The date picker information of the DatePicker content control.
        :rtype: tuple[str | None, str | None, str | None, str | None]
        """

        # Return values
        full_date: str | None = None
        date_format: str | None = None
        language_id: str | None = None
        calendar_type: str | None = None

        # The root of the document
        root: Element = etree.XML(Raw_XML.raw_xml)
        # The body of the document
        body: Element = root.getchildren()[0]

        element: Element
        for element in body.iter(f"{XML_PREFIX}sdt"):
            # Get the nested id tag
            id_tag: Element
            for id_tag in element.iter(f"{XML_PREFIX}id"):
                # If the id matches
                if id_tag.get(f"{XML_PREFIX}val") == self.id:
                    # Get the full date
                    match: Element
                    for match in element.iter(f"{XML_PREFIX}date"):
                        full_date = match.get(f"{XML_PREFIX}fullDate")

                    # Get the date format
                    for match in element.iter(f"{XML_PREFIX}dateFormat"):
                        date_format = match.get(f"{XML_PREFIX}val")

                    # Get the language id
                    for match in element.iter(f"{XML_PREFIX}lid"):
                        language_id = match.get(f"{XML_PREFIX}val")

                    # Get the calendar type
                    for match in element.iter(f"{XML_PREFIX}calendar"):
                        calendar_type = match.get(f"{XML_PREFIX}val")

        # If the date was not found
        return (full_date, date_format, language_id, calendar_type)

    def set_date(self, new_date: datetime, desired_format: str) -> None:
        """
        This method sets the date of a DatePicker content control using a datetime object.
        The format of the date is determined by the date_format property.
        Make sure to supply a valid format string that strftime can use.

        :param new_date: The new date to set.
        :type new_date: datetime
        :param desired_format: The desired format of the date.
        :type desired_format: str
        """

        # set time to 0 to avoid issues with timezones, then format
        date_to_display = new_date.replace(hour=0, minute=0, second=0, microsecond=0)

        # The root of the document
        root: Element = etree.XML(Raw_XML.raw_xml)
        # The body of the document
        body: Element = root.getchildren()[0]

        element: Element
        for element in body.iter(f"{XML_PREFIX}sdt"):
            # Get the nested id tag
            id_tag: Element
            for id_tag in element.iter(f"{XML_PREFIX}id"):
                # If the id matches
                if id_tag.get(f"{XML_PREFIX}val") == self.id:
                    # Set the full date
                    match: Element
                    for match in element.iter(f"{XML_PREFIX}date"):
                        match.set(
                            f"{XML_PREFIX}fullDate",
                            date_to_display.strftime("%Y-%m-%dT%H:%M:%SZ"),
                        )

                    # Set the text
                    content_tag: Element = element.find(f"{XML_PREFIX}sdtContent")

                    p_tag: Element = content_tag.getchildren()[0]
                    p_tag_attributes = p_tag.attrib

                    # Make a new <w:p> tag with the same attributes as the old one
                    new_w_p_tag: Element = etree.Element(
                        f"{XML_PREFIX}p", p_tag_attributes
                    )

                    # Append a <w:r> tag with a <w:t> tag inside it
                    new_w_r_tag: Element = etree.Element(f"{XML_PREFIX}r")
                    new_w_t_tag: Element = etree.Element(f"{XML_PREFIX}t")
                    new_w_t_tag.text = date_to_display.strftime(desired_format)
                    new_w_r_tag.append(new_w_t_tag)
                    new_w_p_tag.append(new_w_r_tag)

                    # Replace the old <w:p> tag with the new one
                    content_tag.replace(p_tag, new_w_p_tag)

        # Update the raw xml
        Raw_XML.raw_xml = etree.tostring(root, encoding="unicode")



try:
    from type_aliases import Element
except ImportError:
    from ..type_aliases import Element


class DocxFormField:
    """
    This is the parent class for all supported form fields.
    """

    def __init__(self, root: Element, file_path: str, name: str) -> None:
        self.root = root
        self.file_path = file_path
        self.name = name



# Local Imports
from .DocxFormField import DocxFormField

try:
    from constants import XML_PREFIX
    from type_aliases import Element
except ImportError:
    from ..constants import XML_PREFIX
    from ..type_aliases import Element

# TODO: Need to look at this a bit more before exposing to user
class CheckBoxFormField(DocxFormField):
    """
    This class holds all information & functions regarding a check box form field.
    """

    def __init__(self, root: Element, file_path: str, name: str) -> None:
        super.__init__(root, file_path, name)
        self.value = self.__get_is_checked()

    def __get_is_checked(self) -> str:
        """
        Return the value of each text tag concatenated
        """
        child: Element
        for child in self.root.iter(f"{XML_PREFIX}ffData"):
            # If > 1 checkbox in a <w:p> tag, we need to find the correct one
            name_tag: Element = child.find(f"{XML_PREFIX}name")
            name_tag_value: str = name_tag.attrib[f"{XML_PREFIX}val"]
            if name_tag_value == self.name:
                for match in child.iter(f"{XML_PREFIX}checked"):
                    return "checked"

        return "unchecked"



try:
    from constants import XML_PREFIX
    from type_aliases import Element
except ImportError:
    from ..constants import XML_PREFIX
    from ..type_aliases import Element


class DocxContentControl:
    """
    This is the parent class of all supported content controls.
    It is responsible for getting the id and text of the content control.
    """

    def __init__(self, root: Element, file_path: str):
        self.root = root
        self.file_path = file_path
        self.id = self.__get_id()
        self.text = self.__get_text()

    def __get_id(self) -> str:
        """
        Return the value of the id tag
        """
        child: Element
        for child in self.root.iter(f"{XML_PREFIX}id"):
            return child.attrib[f"{XML_PREFIX}val"]

        # TODO: Raise an error if the id is not found
        return "NOT FOUND"

    def __get_text(self) -> str:
        """
        Return the value of each text tag concatenated
        """
        child: Element
        tag_text: str = ""
        for child in self.root.iter(f"{XML_PREFIX}t"):
            tag_text += child.text

        return tag_text



# Local Imports
from .DocxContentControl import DocxContentControl

# Package Imports
from lxml import etree

try:
    from type_aliases import Element
    from constants import XML_PREFIX, XML_CHECK
    from globals import Raw_XML
except ImportError:
    from ..type_aliases import Element
    from ..constants import XML_PREFIX, XML_CHECK
    from ..globals import Raw_XML


class CheckBoxContentControl(DocxContentControl):
    """
    This class contains the properties and functions associated with the Check Box content control.

    :param DocxContentControl: This class extends DocxContentControl
    """

    def __init__(self, root: Element, file_path: str):
        super().__init__(root, file_path)

    def set_checkBox(self, check_value: bool):
        """
        This method sets a checkbox to checked or unchecked on a CheckBox Content Control

        :param bool check_value: Sets checkbox to checked for true and unchecked for false
        """

        # Set values according to the boolean passed in
        if check_value == True:
            checkBoxVal = "1"
            box = "☒"
        else:
            checkBoxVal = "0"
            box = "☐"

        # The root of the document
        root: Element = etree.XML(Raw_XML.raw_xml)
        # The body of the document
        body: Element = root.getchildren()[0]

        element: Element
        for element in body.iter(f"{XML_PREFIX}sdt"):
            # Get the nested id tag
            id_tag: Element
            for id_tag in element.iter(f"{XML_PREFIX}id"):
                # If the id matches
                if id_tag.get(f"{XML_PREFIX}val") == self.id:
                    # Set attribute value
                    match: Element
                    for match in element.iter(f"{XML_CHECK}checked"):
                        match.set(f"{XML_CHECK}val", checkBoxVal)

                    # Replace the text
                    for match in element.iter(f"{XML_PREFIX}t"):
                        match.text = box

        # Write the new document to raw_xml
        Raw_XML.raw_xml = etree.tostring(root)



from enum import Enum


class TagType(Enum):
    """
    This enum contains the two types of parent tags that the package is looking for.

    :param Enum: This is the generic Enum class from the enum module
    """

    SDT = "sdt"
    P = "p"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['docx_form',
 'docx_form.constants',
 'docx_form.content_controls',
 'docx_form.enums',
 'docx_form.form_fields',
 'docx_form.globals',
 'docx_form.type_aliases']

package_data = \
{'': ['*']}

install_requires = \
['Sphinx>=5.1.1,<6.0.0', 'lxml>=4.9.1,<5.0.0', 'pytest>=7.1.2,<8.0.0']

setup_kwargs = {
    'name': 'docx-form',
    'version': '0.1.5',
    'description': 'DO NOT USE, THIS IS A WORK IN PROGRESS -- A library that allows the editing of form XML components in .docx files.',
    'long_description': '# docx-form\n\n## Description\n\nThis package allows you to easily modify the values of content controls on forms in Microsoft Word with python scripts. This project currently supports reading and editting of the combo box, check box, plain text, datepicker, and dropdown content controls through the package\'s functions. The file being modified can be overwritten with the new values or saved to a new file.\n\n## Setup For Use\n\n#### Requirements:\n\n- Pip\n- Python version 3.10.6 or higher\n\nSimply run "pip install docx-form" to install docx-form and its required dependencies. After pip is installed, all that is left to do is import the project with "from docx_form import DocxForm".\n[Link to Pip project](https://pypi.org/project/docx-form/)\n\n## Setup For Contribution\n\n#### Requirements:\n\n- Pip\n- Python version 3.10.6 or higher\n- Poetry 1.2.0\n\nClone the repository, for those who aren\'t familiar [https://docs.github.com/en/repositories/creating-and-managing-repositories/cloning-a-repository](https://docs.github.com/en/repositories/creating-and-managing-repositories/cloning-a-repository).\n\n## Poetry Installation\n\nFirst install Python using powershell.\nLink to install: [https://python-poetry.org/docs/](https://python-poetry.org/docs/)\nOnce installed, grab poetry application from the poetry script folder and manually add it to your Python scripts folder. Then in the project environment set the interpreter path to the pypoetry virtual environment. Run Poetry install to install all poetry\'s dependecies. For visual studio code, cmd + p is the command to select intepreter.\n\nThen fetch development and pull in changes and make a new branch off development to start contributing.\n',
    'author': 'Reece Bourgeois',
    'author_email': 'reecebourgeois@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



# Package Imports
from lxml import etree

# Local Imports
from .DocxContentControl import DocxContentControl

try:
    from type_aliases import Element
    from constants import XML_PREFIX
    from globals import Raw_XML
except ImportError:
    from ..type_aliases import Element
    from ..constants import XML_PREFIX
    from ..globals import Raw_XML


class PlainTextContentControl(DocxContentControl):
    """
    This class contains all properties and methods for a Plain Text content control.

    :param DocxContentControl: This class extends the DocxContentControl class
    """

    def __init__(self, root: Element, file_path: str):
        super().__init__(root, file_path)

    def set_text(self, new_text: str) -> None:
        """
        This method sets the text of a Plain Text content control.

        :param str new_text: The new text to set
        """

        # The root of the document
        root: Element = etree.XML(Raw_XML.raw_xml)
        # The body of the document
        body: Element = root.getchildren()[0]

        element: Element
        for element in body.iter(f"{XML_PREFIX}sdt"):
            # Get the nested id tag
            id_tag: Element
            for id_tag in element.iter(f"{XML_PREFIX}id"):
                # If the id matches
                if id_tag.get(f"{XML_PREFIX}val") == self.id:
                    # Replace the text
                    content_tag: Element = element.find(f"{XML_PREFIX}sdtContent")

                    # TODO: Test this in case it is not a universal index
                    p_tag: Element = content_tag.getchildren()[0]
                    p_tag_attributes = p_tag.attrib

                    # Make a new <w:p> tag with the same attributes as the old one
                    new_w_p_tag: Element = etree.Element(
                        f"{XML_PREFIX}p", p_tag_attributes
                    )

                    # Append a <w:r> tag with a <w:t> tag inside it
                    new_w_r_tag: Element = etree.Element(f"{XML_PREFIX}r")
                    new_w_t_tag: Element = etree.Element(f"{XML_PREFIX}t")
                    new_w_t_tag.text = new_text
                    new_w_r_tag.append(new_w_t_tag)
                    new_w_p_tag.append(new_w_r_tag)

                    # Replace the old <w:p> tag with the new one
                    content_tag.replace(p_tag, new_w_p_tag)

        # Write the new document to raw_xml
        Raw_XML.raw_xml = etree.tostring(root)



from .CheckBoxContentControl import CheckBoxContentControl
from .ComboBoxContentControl import ComboBoxContentControl
from .DatePickerContentControl import DatePickerContentControl
from .DropDownListContentControl import DropDownListContentControl
from .PlainTextContentControl import PlainTextContentControl
from .RichTextContentControl import RichTextContentControl
from .DocxContentControl import DocxContentControl